package cn.breeze.elleai.controller.front;


import cn.breeze.elleai.application.dto.ApiResponse;
import cn.breeze.elleai.application.dto.PageResult;
import cn.breeze.elleai.application.dto.inner.AiChatComplateResultDto;
import cn.breeze.elleai.application.dto.request.QaAssistantRequestDto;
import cn.breeze.elleai.application.dto.request.SwitchAssistantMobileRequestDto;
import cn.breeze.elleai.application.dto.request.UserQaHistoryRequestDto;
import cn.breeze.elleai.application.dto.request.UserQaMobileRequestDto;
import cn.breeze.elleai.application.dto.response.*;
import cn.breeze.elleai.application.service.AppChatCompletionService;
import cn.breeze.elleai.application.service.AppCommonService;
import cn.breeze.elleai.config.QueryParam;
import cn.breeze.elleai.util.UserPrincipal;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.Parameter;
import io.swagger.v3.oas.annotations.media.Schema;
import io.swagger.v3.oas.annotations.tags.Tag;
import lombok.RequiredArgsConstructor;
import org.apache.catalina.LifecycleState;
import org.springframework.http.MediaType;
import org.springframework.http.codec.ServerSentEvent;
import org.springframework.web.bind.annotation.*;
import reactor.core.publisher.Flux;

import java.util.List;

@RestController
@RequestMapping(value = "/front/chat")
@Tag(name = "移动端-AI问答")
@RequiredArgsConstructor
public class ChatCompletionMobileController {

    private final AppChatCompletionService chatCompletionService;
    private final AppCommonService commonService;

    @Operation(summary = "助手列表")
    @GetMapping("/assistant/list")
    public ApiResponse<PageResult<QaAssistantDto>> assistantList(@QueryParam QaAssistantRequestDto request) {
        PageResult<QaAssistantDto> pageResult = chatCompletionService.qaAssistantPaginQuery(request);
        return ApiResponse.ok(pageResult);
    }

    @Operation(summary = "热门提问")
    @GetMapping("/hot/list")
    public ApiResponse<PageResult<HotQaMobileDto>> hotList(@QueryParam QaAssistantRequestDto request) {
        return ApiResponse.ok(chatCompletionService.hotQaList(request));
    }

    @Operation(summary = "历史对话")
    @GetMapping("/history")
    public ApiResponse<PageResult<UserChatCompletionMobileDto>> history(@Parameter(hidden = true) UserPrincipal userPrincipal,
                                                                        @QueryParam UserQaHistoryRequestDto request) {

        PageResult<UserChatCompletionMobileDto> pageResult = chatCompletionService.userQaHistoryPaginQuery(userPrincipal, request);
        return ApiResponse.ok(pageResult);
    }


    @Operation(summary = "用户提问")
    @PostMapping("/ask_stream")
    public Flux<ServerSentEvent<UserAskResultMobileDto>> askStream(@Parameter(hidden = true) UserPrincipal userPrincipal,
                                              @RequestBody UserQaMobileRequestDto request) {
        request.setContent(commonService.sentenceWordCorrect(request.getContent()));
        return chatCompletionService.userAsk4Stream2(userPrincipal, request);
    }

    @Operation(summary = "切换助手")
    @PostMapping("/assistant/switch")
    public ApiResponse<String> switchAssistant(@Parameter(hidden = true) UserPrincipal userPrincipal,
                                               @RequestBody SwitchAssistantMobileRequestDto request) {

        return ApiResponse.ok("ok");
    }

    @Operation(summary = "开启新会话，返回新会话ID")
    @PostMapping("/new_completion")
    public ApiResponse<NewCompletionResultMobileDto> newChatCompletion(@Parameter(hidden = true) UserPrincipal userPrincipal) {
        NewCompletionResultMobileDto result = chatCompletionService.newChatCompletion(userPrincipal);
        return ApiResponse.ok(result);
    }

    @Operation(summary = "历史对话详情")
    @GetMapping("/history/detail")
    public ApiResponse<PageResult<UserChatCompletionHistoryMobileDto>> historyDetail(@Parameter(hidden = true) UserPrincipal userPrincipal,
                                                                                     @Schema(description = "历史会话ID") @RequestParam("chat_completion_id") Integer chatCompletionId,
                                                                                     @Schema(description = "页码") @RequestParam("page_no") Integer pageNo,
                                                                                     @Schema(description = "分页数量") @RequestParam("page_size") Integer pageSize) {

        PageResult<UserChatCompletionHistoryMobileDto> pageResult = chatCompletionService.userQaHistoryMobilePaginQuery(userPrincipal, chatCompletionId, pageNo, pageSize);
        return ApiResponse.ok(pageResult);
    }

    @Operation(summary = "用户提问")
    @PostMapping("/ask")
    public ApiResponse<UserAskResultMobileDto> ask(@Parameter(hidden = true) UserPrincipal userPrincipal,
                                                   @RequestBody UserQaMobileRequestDto request) {
        request.setContent(commonService.sentenceWordCorrect(request.getContent()));
        UserAskResultMobileDto result = chatCompletionService.userAsk(userPrincipal, request);
        return ApiResponse.ok(result);
    }

    @Operation(summary = "踩/取消踩")
    @PostMapping("/diss/{message_id}/{status}")
    public ApiResponse<String> diss(@Schema(description = "消息ID") @PathVariable("message_id") Integer messageId,
                                    @Schema(description = "状态（0取消 1踩）") @PathVariable("status") Integer status) {
        chatCompletionService.updateDissStatus(messageId, status);
        return ApiResponse.ok("ok");
    }

    @Operation(summary = "赞/取消赞")
    @PostMapping("/likes/{message_id}/{status}")
    public ApiResponse<String> likes(@Schema(description = "消息ID") @PathVariable("message_id") Integer messageId,
                                     @Schema(description = "状态（0取消 1赞）") @PathVariable("status") Integer status) {
        chatCompletionService.updateLikesStatus(messageId, status);
        return ApiResponse.ok("ok");
    }


    @GetMapping("/kb/segment")
    public ApiResponse<List<String>> getKbSegment(@RequestParam(value = "segment_id") String segmentId) {
        return ApiResponse.ok(chatCompletionService.getKbSegment(segmentId));
    }
}
