package cn.breeze.elleai.domain.sparring.service;

import cn.breeze.elleai.domain.sparring.model.request.*;
import cn.breeze.elleai.domain.sparring.model.response.HotQaResponseModel;
import cn.breeze.elleai.domain.sparring.model.response.QaAssistantResponseModel;
import cn.breeze.elleai.domain.sparring.model.response.UserChatCompletionHistoryResponseModel;
import cn.breeze.elleai.domain.sparring.model.response.UserChatCompletionResponseModel;
import cn.breeze.elleai.infra.entity.HotQaEntity;
import cn.breeze.elleai.infra.entity.QaAssistantEntity;
import cn.breeze.elleai.infra.entity.UserChatCompletionEntity;
import cn.breeze.elleai.infra.entity.UserChatCompletionHistoryEntity;
import cn.breeze.elleai.infra.mapper.HotQaMapper;
import cn.breeze.elleai.infra.mapper.QaAssistantMapper;
import cn.breeze.elleai.infra.mapper.UserChatCompletionHistoryMapper;
import cn.breeze.elleai.infra.mapper.UserChatCompletionMapper;
import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.mybatisflex.core.paginate.Page;
import com.mybatisflex.core.query.QueryWrapper;
import com.mybatisflex.core.update.UpdateChain;
import lombok.RequiredArgsConstructor;
import org.springframework.stereotype.Service;

import java.util.Date;
import java.util.List;
import java.util.Objects;
import java.util.Set;

import static cn.breeze.elleai.infra.entity.table.QaAssistantTableDef.QA_ASSISTANT_ENTITY;
import static cn.breeze.elleai.infra.entity.table.UserChatCompletionHistoryTableDef.USER_CHAT_COMPLETION_HISTORY_ENTITY;
import static cn.breeze.elleai.infra.entity.table.UserChatCompletionTableDef.USER_CHAT_COMPLETION_ENTITY;
import static cn.breeze.elleai.infra.entity.table.HotQaTableDef.HOT_QA_ENTITY;


@Service
@RequiredArgsConstructor
public class ChatCompletionServiceImpl implements ChatCompletionService{

    private final UserChatCompletionHistoryMapper chatCompletionHistoryMapper;

    private final UserChatCompletionMapper chatCompletionMapper;

    private final QaAssistantMapper qaAssistantMapper;

    private final HotQaMapper hotQaMapper;

    @Override
    public Page<UserChatCompletionResponseModel> userQaPaginQuery(UserQaRequestModel request) {
        Integer pageNo = ObjectUtil.defaultIfNull(request.getPageNo(), 1);
        Integer pageSize = ObjectUtil.defaultIfNull(request.getPageSize(), 10);
        QueryWrapper queryWrapper = QueryWrapper.create();
        if(StrUtil.isNotEmpty(request.getUserName())) {
            queryWrapper.where(USER_CHAT_COMPLETION_ENTITY.USER_NAME.eq(request.getUserName()));
        }
        if(StrUtil.isNotEmpty(request.getShopName())) {
            queryWrapper.where(USER_CHAT_COMPLETION_ENTITY.SHOP_NAME.like("%"+request.getShopName()+"%"));
        }
        if(Objects.nonNull(request.getStartTime())) {
            queryWrapper.where(USER_CHAT_COMPLETION_ENTITY.CREATE_TIME.ge(request.getStartTime()));
        }
        if(Objects.nonNull(request.getEndTime())) {
            queryWrapper.where(USER_CHAT_COMPLETION_ENTITY.CREATE_TIME.le(request.getEndTime()));
        }
        queryWrapper.orderBy(USER_CHAT_COMPLETION_ENTITY.CREATE_TIME, false);
        return  chatCompletionMapper.paginateAs(pageNo, pageSize, queryWrapper, UserChatCompletionResponseModel.class);
    }

    @Override
    public Page<UserChatCompletionHistoryResponseModel> userQaHistoryPaginQuery(UserQaHistoryRequestModel request) {
        Integer pageNo = ObjectUtil.defaultIfNull(request.getPageNo(), 1);
        Integer pageSize = ObjectUtil.defaultIfNull(request.getPageSize(), 10);

        QueryWrapper queryWrapper = QueryWrapper.create();
        if(Objects.nonNull(request.getChatCompletionId())) {
            queryWrapper.where(USER_CHAT_COMPLETION_HISTORY_ENTITY.CHAT_COMPLETION_ID.eq(request.getChatCompletionId()));
        }
        if(CollectionUtil.isNotEmpty(request.getChatCompletionIdList())) {
            queryWrapper.where(USER_CHAT_COMPLETION_HISTORY_ENTITY.CHAT_COMPLETION_ID.in(request.getChatCompletionIdList()));
        }
        if(Objects.nonNull(request.getType())) {
            queryWrapper.where(USER_CHAT_COMPLETION_HISTORY_ENTITY.TYPE.eq(request.getType()));
        }
        queryWrapper.orderBy(USER_CHAT_COMPLETION_HISTORY_ENTITY.CREATE_TIME, false);

        Page<UserChatCompletionHistoryResponseModel> page = chatCompletionHistoryMapper.paginateAs(pageNo, pageSize, queryWrapper, UserChatCompletionHistoryResponseModel.class);

        return page;
    }

    @Override
    public List<UserChatCompletionResponseModel> userQaList(UserQaRequestModel request) {
        QueryWrapper queryWrapper = QueryWrapper.create();
        if(StrUtil.isNotEmpty(request.getUserId())) {
            queryWrapper.where(USER_CHAT_COMPLETION_ENTITY.USER_ID.eq(request.getUserId()));
        }
        return chatCompletionMapper.selectListByQueryAs(queryWrapper, UserChatCompletionResponseModel.class);
    }

    @Override
    public List<UserChatCompletionHistoryResponseModel> userQaHistoryDetail(Integer chatCompletionId) {
        QueryWrapper queryWrapper = QueryWrapper.create();
        queryWrapper.where(USER_CHAT_COMPLETION_HISTORY_ENTITY.CHAT_COMPLETION_ID.eq(chatCompletionId));
        queryWrapper.orderBy(USER_CHAT_COMPLETION_HISTORY_ENTITY.CREATE_TIME, true);

        return chatCompletionHistoryMapper.selectListByQueryAs(queryWrapper, UserChatCompletionHistoryResponseModel.class);
    }

    @Override
    public UserChatCompletionHistoryResponseModel getReplyMessageByUserMessageId(Integer userMessageId) {
        QueryWrapper queryWrapper = QueryWrapper.create();
        queryWrapper.where(USER_CHAT_COMPLETION_HISTORY_ENTITY.REPLY_MESSAGE_ID.eq(userMessageId));

        return chatCompletionHistoryMapper.selectOneByQueryAs(queryWrapper, UserChatCompletionHistoryResponseModel.class);
    }

    @Override
    public Page<UserChatCompletionHistoryResponseModel> userQaHistoryDetailPaginQuery(Integer chatCompletionId, Integer pageNo, Integer pageSize) {

        QueryWrapper queryWrapper = QueryWrapper.create();
        queryWrapper.where(USER_CHAT_COMPLETION_HISTORY_ENTITY.CHAT_COMPLETION_ID.eq(chatCompletionId));
        queryWrapper.orderBy(USER_CHAT_COMPLETION_HISTORY_ENTITY.CREATE_TIME, true);

        Page<UserChatCompletionHistoryResponseModel> page = chatCompletionHistoryMapper.paginateAs(pageNo, pageSize, queryWrapper, UserChatCompletionHistoryResponseModel.class);

        return page;
    }

    @Override
    public UserChatCompletionResponseModel userQaDetail(UserQaRequestModel request) {
        QueryWrapper queryWrapper = QueryWrapper.create();
        if(Objects.nonNull(request.getUserId())) {
            queryWrapper.where(USER_CHAT_COMPLETION_ENTITY.USER_ID.eq(request.getUserId()));
        }
        if(Objects.nonNull(request.getChatCompletionId())) {
            queryWrapper.where(USER_CHAT_COMPLETION_ENTITY.ID.eq(request.getChatCompletionId()));
        }
        queryWrapper.orderBy(USER_CHAT_COMPLETION_ENTITY.CREATE_TIME, false);
        return chatCompletionMapper.selectOneByQueryAs(queryWrapper, UserChatCompletionResponseModel.class);
    }

    @Override
    public Integer saveUserQaSession(UserChatCompletionSaveModel dto) {
        UserChatCompletionEntity entity = new UserChatCompletionEntity();
        BeanUtil.copyProperties(dto, entity);
        entity.setUpdateTime(new Date());
        chatCompletionMapper.insertOrUpdateSelective(entity);
        return entity.getId();
    }

    @Override
    public Integer saveUserQaRecord(Integer chatCompletionId, Integer type, Integer replyMessageId, String content, Integer assistantId, String recordId) {
        UserChatCompletionHistoryEntity entity = new UserChatCompletionHistoryEntity();
        entity.setChatCompletionId(chatCompletionId);
        entity.setType(type);
        entity.setReplyMessageId(replyMessageId);
        entity.setContent(content);
        entity.setAssistantId(assistantId);
        entity.setRecordId(recordId);
        entity.setCreateTime(new Date());
        chatCompletionHistoryMapper.insertOrUpdateSelective(entity);
        return entity.getId();
    }

    @Override
    public Integer saveUserQaRecord(UserQaHistorySaveModel dto) {
        UserChatCompletionHistoryEntity entity = new UserChatCompletionHistoryEntity();
        BeanUtil.copyProperties(dto, entity);
        chatCompletionHistoryMapper.insertOrUpdateSelective(entity);
        return entity.getId();
    }

    @Override
    public Page<QaAssistantResponseModel> qaAssistantPaginQuery(QaAssistantRequestModel request) {

        Integer pageNo = ObjectUtil.defaultIfNull(request.getPageNo(), 1);
        Integer pageSize = ObjectUtil.defaultIfNull(request.getPageSize(), 10);

        QueryWrapper queryWrapper = QueryWrapper.create()
                .where(QA_ASSISTANT_ENTITY.DELETED.eq(0));
        if(StrUtil.isNotEmpty(request.getName())) {
            queryWrapper.and(QA_ASSISTANT_ENTITY.NAME.like("%"+request.getName()+"%"));
        }
        if(Objects.nonNull(request.getStatus())) {
            queryWrapper.and(QA_ASSISTANT_ENTITY.STATUS.eq(request.getStatus()));
        }
        queryWrapper.orderBy(QA_ASSISTANT_ENTITY.CREATE_TIME, true);

        Page<QaAssistantResponseModel> page = qaAssistantMapper.paginateAs(pageNo, pageSize, queryWrapper, QaAssistantResponseModel.class);

        return page;
    }

    @Override
    public List<QaAssistantResponseModel> qaAssistantList(QaAssistantRequestModel request) {
        QueryWrapper queryWrapper = QueryWrapper.create()
                .where(QA_ASSISTANT_ENTITY.DELETED.eq(0));
        if(StrUtil.isNotEmpty(request.getName())) {
            queryWrapper.where(QA_ASSISTANT_ENTITY.NAME.like("%"+request.getName()+"%"));
        }
        queryWrapper.orderBy(QA_ASSISTANT_ENTITY.CREATE_TIME, false);

        return qaAssistantMapper.selectListByQueryAs(queryWrapper, QaAssistantResponseModel.class);
    }

    @Override
    public QaAssistantResponseModel qaAssistantDetail(Integer id) {
        QueryWrapper queryWrapper = QueryWrapper.create()
                .where(QA_ASSISTANT_ENTITY.ID.eq(id).and(QA_ASSISTANT_ENTITY.DELETED.eq(0)));
        return qaAssistantMapper.selectOneByQueryAs(queryWrapper, QaAssistantResponseModel.class);
    }

    @Override
    public void updateQaAssistantStatus(Integer id, Integer status) {
        UpdateChain.of(QaAssistantEntity.class)
                .set(QA_ASSISTANT_ENTITY.STATUS, status)
                .set(QA_ASSISTANT_ENTITY.UPDATE_TIME, new Date())
                .where(QA_ASSISTANT_ENTITY.ID.eq(id))
                .update();
    }

    @Override
    public void deleteQaAssistant(Integer id) {
        UpdateChain.of(QaAssistantEntity.class)
                .set(QA_ASSISTANT_ENTITY.DELETED, 1)
                .set(QA_ASSISTANT_ENTITY.UPDATE_TIME, new Date())
                .where(QA_ASSISTANT_ENTITY.ID.eq(id))
                .update();
    }

    @Override
    public void saveQaAssistant(QaAssistantSaveModel dto) {
        QaAssistantEntity entity = BeanUtil.toBean(dto, QaAssistantEntity.class);
        entity.setUpdateTime(new Date());
        if(Objects.isNull(entity.getId())) {
            entity.setStatus(0);
            entity.setDeleted(0);
        }
        qaAssistantMapper.insertOrUpdateSelective(entity);
    }

    @Override
    public Page<HotQaResponseModel> hotQaPaginQuery(HotQaRequestModel request) {
        Integer pageNo = ObjectUtil.defaultIfNull(request.getPageNo(), 1);
        Integer pageSize = ObjectUtil.defaultIfNull(request.getPageSize(), 10);

        QueryWrapper queryWrapper = QueryWrapper.create()
                .where(HOT_QA_ENTITY.DELETED.eq(0));
        if(StrUtil.isNotEmpty(request.getName())) {
            queryWrapper.and(HOT_QA_ENTITY.QUESTION.like("%"+request.getName()+"%"));
        }
        if(Objects.nonNull(request.getStatus())) {
            queryWrapper.and(HOT_QA_ENTITY.STATUS.eq(request.getStatus()));
        }
        queryWrapper.orderBy(HOT_QA_ENTITY.SORT_INDEX, false);

        Page<HotQaResponseModel> page = hotQaMapper.paginateAs(pageNo, pageSize, queryWrapper, HotQaResponseModel.class);

        return page;
    }

    @Override
    public List<HotQaResponseModel> hotQaList(HotQaRequestModel request) {
        QueryWrapper queryWrapper = QueryWrapper.create()
                .where(HOT_QA_ENTITY.DELETED.eq(0));

        if (ObjectUtil.isNotNull(request.getAssistantId())) {
            queryWrapper.and(HOT_QA_ENTITY.ASSISTANT_ID.eq(request.getAssistantId()));
        }
        if(StrUtil.isNotEmpty(request.getName())) {
            queryWrapper.and(HOT_QA_ENTITY.QUESTION.like("%"+request.getName()+"%"));
        }
        if(Objects.nonNull(request.getStatus())) {
            queryWrapper.and(HOT_QA_ENTITY.STATUS.eq(request.getStatus()));
        }
        queryWrapper.orderBy(HOT_QA_ENTITY.SORT_INDEX, false);

        return hotQaMapper.selectListByQueryAs(queryWrapper, HotQaResponseModel.class);
    }

    @Override
    public HotQaResponseModel hotQaDetail(Integer id) {
        QueryWrapper queryWrapper = QueryWrapper.create()
                .where(HOT_QA_ENTITY.ID.eq(id).and(HOT_QA_ENTITY.DELETED.eq(0)));
        return hotQaMapper.selectOneByQueryAs(queryWrapper, HotQaResponseModel.class);
    }

    @Override
    public void updateHotQaStatus(Integer id, Integer status) {
        UpdateChain.of(HotQaEntity.class)
                .set(HOT_QA_ENTITY.STATUS, status)
                .where(HOT_QA_ENTITY.ID.eq(id))
                .update();
    }

    @Override
    public void deleteHotQa(Integer id) {
        UpdateChain.of(HotQaEntity.class)
                .set(HOT_QA_ENTITY.DELETED, 1)
                .where(HOT_QA_ENTITY.ID.eq(id))
                .update();
    }

    @Override
    public void saveHotQa(HotQaSaveModel dto) {
        HotQaEntity entity = BeanUtil.toBean(dto, HotQaEntity.class);
        hotQaMapper.insertOrUpdateSelective(entity);
    }

    @Override
    public List<QaAssistantResponseModel> qaAssistantListByIds(Set<Integer> assistantIds) {
        QueryWrapper queryWrapper = QueryWrapper.create()
                .where(QA_ASSISTANT_ENTITY.ID.in(assistantIds));
        return qaAssistantMapper.selectListByQueryAs(queryWrapper, QaAssistantResponseModel.class);
    }
}
