package cn.breeze.elleai.facade;

import cn.breeze.elleai.application.dto.langchain.EmbeddingRequest;
import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import dev.langchain4j.data.embedding.Embedding;
import dev.langchain4j.data.segment.TextSegment;
import dev.langchain4j.model.openai.OpenAiEmbeddingModel;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;

import java.time.Duration;
import java.util.List;
import java.util.concurrent.ConcurrentHashMap;
import java.util.concurrent.ConcurrentMap;

/**
 * @author yangyw
 */
@Component
@Slf4j
@RequiredArgsConstructor
public class OpenAIEmbeddingFacade implements EmbeddingService {

    @Value("${embedding.api-base-url:https://elle.e-tools.cn/v1}")
    private String apiBaseUrl;

    @Value("${embedding.api-key:smartbreeze}")
    private String apiKey;

    @Value("${embedding.model-name:bge-m3}")
    private String modelName;

    private final ConcurrentMap<String, OpenAiEmbeddingModel> modelMap = new ConcurrentHashMap<>();

    /**
     * 获取OpenAiEmbeddingModel
     * @param request
     * @return
     */
    private OpenAiEmbeddingModel getModel(EmbeddingRequest request) {
        return modelMap.compute(request.getApiBaseUrl() + request.getModel(), (k, v) -> {
            if (ObjectUtil.isNull(v)) {
                v = OpenAiEmbeddingModel.builder()
                        .modelName(StrUtil.blankToDefault(request.getModel(), modelName))
                        .baseUrl(StrUtil.blankToDefault(request.getApiBaseUrl(), apiBaseUrl))
                        .apiKey(StrUtil.blankToDefault(request.getApiKey(), apiKey))
                        .timeout(Duration.ofSeconds(30))
                        .build();
            }
            return v;
        });
    }

    @Override
    public List<Embedding> embed(List<String> texts) {
        EmbeddingRequest embeddingRequest = new EmbeddingRequest();
        embeddingRequest.setTexts(texts);
        return embed(embeddingRequest);
    }

    @Override
    public Embedding embed(String text) {
        return CollUtil.getFirst(embed(List.of(text)));
    }

    @Override
    public List<Embedding> embed(EmbeddingRequest request) {
        return getModel(request).embedAll(CollUtil.map(request.getTexts(), TextSegment::from, true)).content();
    }
}
